<cfcomponent name="consoleService" output="false">

	<cffunction name="init" access="public" output="false" returntype="standard.com.xbox.consoleService">
		<cfargument name="consoleDAO" type="standard.com.xbox.consoleDAO" required="true" />
		<cfargument name="consoleGateway" type="standard.com.xbox.consoleGateway" required="true" />
		<cfargument name="controlDAO" type="standard.com.xbox.controls.controlDAO" required="true" />
		<cfargument name="controlGateway" type="standard.com.xbox.controls.controlGateway" required="true" />
		<cfargument name="accessoryDAO" type="standard.com.xbox.accessories.accessoryDAO" required="true" />
		<cfargument name="accessoryGateway" type="standard.com.xbox.accessories.accessoryGateway" required="true" />
		<cfargument name="gameDAO" type="standard.com.xbox.games.gameDAO" required="true" />
		<cfargument name="gameGateway" type="standard.com.xbox.games.gameGateway" required="true" />

		<cfset variables.consoleDAO = arguments.consoleDAO />
		<cfset variables.consoleGateway = arguments.consoleGateway />
		<cfset variables.controlDAO = arguments.controlDAO />
		<cfset variables.controlGateway = arguments.controlGateway />
		<cfset variables.accessoryDAO = arguments.accessoryDAO />
		<cfset variables.accessoryGateway = arguments.accessoryGateway />
		<cfset variables.gameDAO = arguments.gameDAO />
		<cfset variables.gameGateway = arguments.gameGateway />

		<cfreturn this/>
	</cffunction>

<!--- console --->
	<cffunction name="createConsole" access="public" output="false" returntype="console">
		<cfargument name="consoleID" type="String" required="true" />
		<cfargument name="type" type="String" required="false" />
		<cfargument name="storage" type="Numeric" required="false" />
		
		<cfset var console = createObject("component","standard.com.xbox.console").init(argumentCollection=arguments) />
		<cfreturn console />
	</cffunction>
	<cffunction name="getConsole" access="public" output="false" returntype="console">
		<cfargument name="consoleID" type="String" required="true" />
		
		<cfset var console = createConsole(argumentCollection=arguments) />
		<cfset variables.consoleDAO.read(console) />
		<!--- get relationships and pass them in --->
		<cfset console.setAccessories(getAccessories(consoleID=console.getConsoleID())) />
		<cfset console.setGames(getGames(consoleID=console.getConsoleID())) />
		<cfset console.setControls(getControls(consoleID=console.getConsoleID())) />
		<cfreturn console />
	</cffunction>
	<cffunction name="getConsoles" access="public" output="false" returntype="array">
		<cfargument name="consoleID" type="String" required="false" />
		<cfargument name="type" type="String" required="false" />
		<cfargument name="storage" type="Numeric" required="false" />
		
		<cfreturn variables.consoleGateway.getByAttributes(argumentCollection=arguments) />
	</cffunction>
	<cffunction name="saveConsole" access="public" output="false" returntype="boolean">
		<cfargument name="console" type="console" required="true" />
		
		<cfreturn variables.consoleDAO.save(arguments.console) />
	</cffunction>
	<cffunction name="deleteConsole" access="public" output="false" returntype="boolean">
		<cfargument name="consoleID" type="String" required="true" />
		
		<cfset var console = createConsole(argumentCollection=arguments) />
		<cfreturn variables.consoleDAO.delete(console) />
	</cffunction>

<!--- control --->
	<cffunction name="createControl" access="public" output="false" returntype="standard.com.xbox.controls.control">
		<cfargument name="controlID" type="uuid" required="true" />
		<cfargument name="wireless" type="Boolean" required="false" />
		<cfargument name="headset" type="Boolean" required="false" />
		
		<cfset var control = createObject("component","standard.com.xbox.controls.control").init(argumentCollection=arguments) />
		<cfreturn control />
	</cffunction>
	<cffunction name="getControl" access="public" output="false" returntype="control">
		<cfargument name="controlID" type="uuid" required="true" />
		
		<cfset var control = createControl(argumentCollection=arguments) />
		<cfset variables.controlDAO.read(control) />
		<cfreturn control />
	</cffunction>
	<cffunction name="getControls" access="public" output="false" returntype="array">
		<cfargument name="controlID" type="uuid" required="false" />
		<cfargument name="wireless" type="Boolean" required="false" />
		<cfargument name="headset" type="Boolean" required="false" />
		<cfargument name="consoleID" type="uuid" required="false" />
		
		<cfreturn variables.controlGateway.getByAttributes(argumentCollection=arguments) />
	</cffunction>
	<cffunction name="saveControl" access="public" output="false" returntype="boolean">
		<cfargument name="control" type="standard.com.xbox.controls.control" required="true" />
		
		<cfreturn variables.controlDAO.save(arguments.control) />
	</cffunction>
	<cffunction name="deleteControl" access="public" output="false" returntype="boolean">
		<cfargument name="controlID" type="String" required="true" />
		
		<cfset var control = createControl(argumentCollection=arguments) />
		<cfreturn variables.consoleDAO.delete(control) />
	</cffunction>
	
<!--- game --->
	<cffunction name="createGame" access="public" output="false" returntype="standard.com.xbox.games.game">
		<cfargument name="gameID" type="uuid" required="true" />
		<cfargument name="gameName" type="String" required="false" />
		<cfargument name="specialEdition" type="Boolean" required="false" />
		
		<cfset var game = createObject("component","standard.com.xbox.games.game").init(argumentCollection=arguments) />
		<cfreturn game />
	</cffunction>
	<cffunction name="getGame" access="public" output="false" returntype="standard.com.xbox.games.game">
		<cfargument name="gameID" type="uuid" required="true" />
		
		<cfset var game = createGame(argumentCollection=arguments) />
		<cfset variables.gameDAO.read(game) />
		<cfreturn game />
	</cffunction>
	<cffunction name="getGames" access="public" output="false" returntype="array">
		<cfargument name="gameID" type="uuid" required="false" />
		<cfargument name="gameName" type="String" required="false" />
		<cfargument name="specialEdition" type="Boolean" required="false" />
		<cfargument name="consoleID" type="uuid" required="false" />
		
		<cfreturn variables.gameGateway.getByAttributes(argumentCollection=arguments) />
	</cffunction>
	<cffunction name="saveGame" access="public" output="false" returntype="boolean">
		<cfargument name="game" type="standard.com.xbox.games.game" required="true" />

		<cfreturn variables.gameDAO.save(arguments.game) />
	</cffunction>
	<cffunction name="deleteGame" access="public" output="false" returntype="boolean">
		<cfargument name="gameID" type="uuid" required="true" />
		
		<cfset var game = createGame(argumentCollection=arguments) />
		<cfreturn variables.gameDAO.delete(game) />
	</cffunction>
	
<!--- accessory --->
	<cffunction name="createAccessory" access="public" output="false" returntype="standard.com.xbox.accessories.accessory">
		<cfargument name="accessoryID" type="uuid" required="true" />
		<cfargument name="accessoryName" type="String" required="false" />
		
		<cfset var accessory = createObject("component","standard.com.xbox.accessories.accessory").init(argumentCollection=arguments) />
		<cfreturn accessory />
	</cffunction>
	<cffunction name="getAccessory" access="public" output="false" returntype="standard.com.xbox.accessories.accessory">
		<cfargument name="accessoryID" type="uuid" required="true" />
		
		<cfset var accessory = createAccessory(argumentCollection=arguments) />
		<cfset variables.accessoryDAO.read(accessory) />
		<cfreturn accessory />
	</cffunction>
	<cffunction name="getAccessories" access="public" output="false" returntype="array">
		<cfargument name="accessoryID" type="uuid" required="false" />
		<cfargument name="accessoryName" type="String" required="false" />
		<cfargument name="consoleID" type="uuid" required="false" />
		
		<cfreturn variables.accessoryGateway.getByAttributes(argumentCollection=arguments) />
	</cffunction>
	<cffunction name="saveAccessory" access="public" output="false" returntype="boolean">
		<cfargument name="accessory" type="standard.com.xbox.accessories.accessory" required="true" />

		<cfreturn variables.accessoryDAO.save(accessory) />
	</cffunction>
	<cffunction name="deleteaccessory" access="public" output="false" returntype="boolean">
		<cfargument name="accessoryID" type="uuid" required="true" />
		
		<cfset var accessory = createAccessory(argumentCollection=arguments) />
		<cfreturn variables.accessoryDAO.delete(accessory) />
	</cffunction>
</cfcomponent>
